@@ -28,36 +28,31 @@
 #include <glib/gi18n.h>
 #include <glib-object.h>
 #include <gio/gio.h>
+#include <gdk/gdk.h>
 #include <gdk/gdkkeysyms.h>
 #include <gtk/gtk.h>
-#include <libgnome/libgnome.h>
-#include <libgnomeui/libgnomeui.h>
+#include <gdk-pixbuf/gdk-pixbuf.h>
 
 #include "gui.h"
 #include "main.h"
 #include "prefs.h"
-#include "menu.h"
 #include "channel.h"
+#include "http.h"
+#include "pixpack.h"
 
 struct _TgGui {
     GObject parent_instance;
 
-    GtkWidget *app;
+    GtkBuilder *builder;
+    GtkWidget *window;
+    GtkWidget *entry;
+    GtkWidget *pixpack;
+    GtkWidget *progress_bar;
+    GtkWidget *status_bar;
 
     GSettings *settings;
 
-    GtkWidget *statusbar;
-    GtkWidget *entry;
-    GtkWidget *pixmap;
-
-    GtkProgressBar *progress;
-
-    GtkWidget *zoomlabel;
-
-    GtkWidget *zoombutton;
-    GtkWidget *pagebutton;
-
-    GtkWidget *channel_menu;
+    GMenu *channel_menu;
 
     /* for timer-input */
     gint logo_timer;
@@ -68,12 +63,14 @@ struct _TgGui {
     gint page_msecs;
     gboolean page_status; /* auto-paging enabled or disabled */
     gint page_progress;
-
-    /* FIXME: Multiple views */
+    gint zoom_factor;
+    gint page_nr;
+    gint subpage_nr;
 
     gchar **channel_children;
     GSList *channels;
-    gchar *current_channel;
+    gchar *current_channel_uuid;
+    TgChannel *current_channel;
 };
 
 enum {
@@ -97,13 +94,13 @@ tg_gui_update_title_bar(void)
 {
     char buf[100];
     /* update the title bar */
-    if (currentview != NULL && currentview->channel != NULL) {
+    if (gui->current_channel != NULL) {
 	gchar *name, *desc;
 	g_object_get(
-	    currentview->channel, "name", &name, "description", &desc, NULL);
+	    gui->current_channel, "name", &name, "description", &desc, NULL);
 	if (name != NULL && desc != NULL) {
 	    sprintf(buf, _("TeleGNOME: %s (%s)"), name, desc);
-	    gtk_window_set_title(GTK_WINDOW(gui->app), buf);
+	    gtk_window_set_title(GTK_WINDOW(gui->window), buf);
 	}
 	g_free(desc);
 	g_free(name);
@@ -115,7 +112,7 @@ static gint
 tg_gui_logo_timer(gpointer g) 
 {
     if (gui->logo_timer != -1)
-	gtk_timeout_remove(gui->logo_timer);
+	g_source_remove(gui->logo_timer);
     gui->logo_timer = -1;
     tg_gui_get_the_page(FALSE);
     return 0;
@@ -144,9 +141,9 @@ tg_gui_channel_select(TgChannel *channel)
 {
     g_assert(channel != NULL);
 
-    if (currentview->channel)
-	g_object_unref(G_OBJECT(currentview->channel));
-    currentview->channel = g_object_ref(G_OBJECT(channel));
+    if (gui->current_channel)
+	g_object_unref(G_OBJECT(gui->current_channel));
+    gui->current_channel = g_object_ref(G_OBJECT(channel));
 
     tg_gui_update_title_bar();
 
@@ -161,58 +158,45 @@ tg_gui_channel_select(TgChannel *channel)
 #endif
 }
 
-static void
-tg_gui_channel_menu_item_activate(GtkWidget *w, gpointer data)
+void
+tg_gui_change_state_set_channel(GSimpleAction *action, GVariant *value,
+				gpointer data)
 {
-    gchar *uuid;
+    const gchar *uuid;
 
-    g_assert(data != NULL);
-    g_object_get(TG_CHANNEL(data), "uuid", &uuid, NULL);
-    g_object_set(
-	gui,
-	"current-channel", uuid,
-	"current-page-number", 100,
-	"current-subpage-number", 0,
-	NULL);
-    g_free(uuid);
+    uuid = g_variant_get_string(value, NULL);
+    g_object_set(gui,
+		 "current-channel", uuid,
+		 "current-page-number", 100,
+		 "current-subpage-number", 0,
+		 NULL);
     tg_gui_get_the_page(FALSE);
 }
 
 /*************************
  * create the channel menu
  */
-static GtkWidget *
-tg_gui_create_channel_menu(void)
+static void
+tg_gui_populate_channel_menu(void)
 {
-    GtkWidget *menu, *item;
     int i;
     TgChannel *channel;
 
     g_assert(gui->channels != NULL);
-    menu = gtk_menu_new();
+
+    g_menu_remove_all(gui->channel_menu);
 
     for (i=0; i<g_slist_length(gui->channels); i++) {
-	gchar *name;
+	gchar *name, *uuid, *action;
 
 	channel = TG_CHANNEL(g_slist_nth_data(gui->channels, i));
-	g_object_get(channel, "name", &name, NULL);
-
-	item = gtk_menu_item_new_with_label(name);
-
-	g_signal_connect(G_OBJECT(item), "activate",
-			 G_CALLBACK(tg_gui_channel_menu_item_activate),
-			 (gpointer)channel);
-	gtk_menu_shell_append(GTK_MENU_SHELL(menu), item);
-	gtk_widget_show(item);
-
+	g_object_get(channel, "name", &name, "uuid", &uuid, NULL);
+	action = g_strdup_printf("app.set-channel::%s", uuid);
+	g_menu_append(gui->channel_menu, name, action);
+	g_free(action);
+	g_free(uuid);
 	g_free(name);
     }
-
-    item = gtk_menu_item_new_with_label(_("Channels"));
-    gtk_widget_show(item);
-    gtk_menu_item_set_submenu(GTK_MENU_ITEM(item), menu);
-
-    return item;
 }
 
 /*************************
@@ -231,8 +215,7 @@ tg_gui_reload_channels(void)
 	gui->channels = NULL;
     }
 
-    if (currentview->channel)
-	g_object_get(currentview->channel, "uuid", &current_uuid, NULL);
+    current_uuid = g_strdup(gui->current_channel_uuid);
 
     for (childp = gui->channel_children; childp && *childp; ++childp) {
 	channel = tg_channel_new(*childp, NULL);
@@ -301,7 +284,7 @@ tg_gui_reload_channels(void)
 
     if (current_uuid)
 	g_object_set(gui, "current-channel", current_uuid, NULL);
-    if (!currentview->channel) {
+    if (!gui->current_channel) {
 	gchar *first_uuid;
 	g_object_get(
 	    TG_CHANNEL(gui->channels->data), "uuid", &first_uuid, NULL);
@@ -321,21 +304,11 @@ tg_gui_reload_channels(void)
 static void
 tg_gui_refresh_channel_menu(void)
 {
-    /* dispose the menu if it was already added */
-    if (gui->channel_menu != NULL) {
-	gtk_container_remove(
-	    GTK_CONTAINER(GNOME_APP(gui->app)->menubar), gui->channel_menu);
-    }
-    
     /* load the channels from disk */
     tg_gui_reload_channels();
 
-    /* create the menu */
-    gui->channel_menu = tg_gui_create_channel_menu();
-    
-    /* and add it to the menu bar */
-    gtk_menu_shell_insert(
-	GTK_MENU_SHELL(GNOME_APP(gui->app)->menubar), gui->channel_menu, 2);
+    /* re-populate the menu */
+    tg_gui_populate_channel_menu();
 }
 
 /*******************************
@@ -344,48 +317,24 @@ tg_gui_refresh_channel_menu(void)
 static void
 tg_gui_print_in_statusbar(const char *buf)  /*FIXME: buffersize*/
 {
-    g_assert(buf != NULL);
-    gnome_appbar_set_status(GNOME_APPBAR(gui->statusbar), buf);
-    gtk_widget_show(GTK_WIDGET(gui->statusbar));
+    guint context_id;
+
+    context_id = gtk_statusbar_get_context_id(GTK_STATUSBAR(gui->status_bar),
+					      "errors");
+    gtk_statusbar_remove_all(GTK_STATUSBAR(gui->status_bar), context_id);
+    if (buf)
+	gtk_statusbar_push(GTK_STATUSBAR(gui->status_bar), context_id, buf);
 }
 
-/******************************* 
- * create a new entry 
- */
-static GtkWidget * 
-tg_gui_new_entry (void)
-{
-	GtkWidget *entry = NULL;
-
-	entry=gtk_entry_new();
-	gtk_entry_set_max_length(GTK_ENTRY(entry),
-				 TG_PAGE_SIZE + 1 + TG_SUBPAGE_SIZE);
-	
-	/* hack */
-	gtk_widget_set_usize(GTK_WIDGET(entry),
-			     (8 * (TG_PAGE_SIZE + 1 + TG_SUBPAGE_SIZE)),20);
-	
-	/*gtk_entry_set_text(GTK_ENTRY(entry), startpage);*/
-	      
-	g_signal_connect(G_OBJECT(entry), "activate", 
-			 G_CALLBACK(tg_gui_cb_goto_page), NULL);
-
-	/* save entry for later ref */
-	gui->entry = entry;
-
-	return entry;
-}
-
-
 static gint
 tg_gui_pager_timer(gpointer g)
 {
     gui->page_progress += gui->page_msecs/100;
-    gtk_progress_bar_set_fraction(gui->progress, gui->page_progress / (gdouble)gui->page_msecs);
+    gtk_progress_bar_set_fraction(GTK_PROGRESS_BAR(gui->progress_bar), gui->page_progress / (gdouble)gui->page_msecs);
 
     if (gui->page_progress >= gui->page_msecs) {
 	gui->page_progress = 0;
-	gtk_progress_bar_set_fraction(gui->progress, 0.0);
+	gtk_progress_bar_set_fraction(GTK_PROGRESS_BAR(gui->progress_bar), 0.0);
 	tg_gui_cb_next_page(NULL, NULL);
     }
     return 1;
@@ -394,92 +343,20 @@ tg_gui_pager_timer(gpointer g)
 static void 
 tg_gui_cb_toggle_paging(GtkWidget *w, gpointer data) 
 {
-    gtk_progress_bar_set_fraction(gui->progress, 0.0);
+    gtk_progress_bar_set_fraction(GTK_PROGRESS_BAR(gui->progress_bar), 0.0);
     if (gui->page_status) {
-	if (gui->page_timer != -1) gtk_timeout_remove(gui->page_timer);
+	if (gui->page_timer != -1)
+	    g_source_remove(gui->page_timer);
 	gui->page_timer = -1;
 	gui->page_status = FALSE;
 	gui->page_progress = 0;
     } else {
 	gui->page_progress = 0;
 	gui->page_status = TRUE;
-	gui->page_timer = gtk_timeout_add(gui->page_msecs/100, tg_gui_pager_timer, NULL);
+	gui->page_timer = g_timeout_add(gui->page_msecs/100, tg_gui_pager_timer, NULL);
     }
 }
 
-/*******************************
- * create a new toolbar 
- */
-static GtkWidget *
-tg_gui_new_toolbar (void)
-{
-    GtkWidget *icon, *toolbar, *entry, *hbox, *w;
-    
-    toolbar= gtk_toolbar_new();
-
-    hbox = gtk_hbox_new(FALSE, 0);
-    
-    w = gtk_label_new(_("Page:"));
-    gtk_box_pack_start(GTK_BOX(hbox), w, FALSE, FALSE, 5);
-    
-    /* add the entry */
-    entry = tg_gui_new_entry();
-    gtk_widget_set_tooltip_text(entry, _("Page number"));
-
-    gtk_box_pack_start(GTK_BOX(hbox), entry, FALSE, FALSE, 5);
-    gtk_toolbar_append_widget (GTK_TOOLBAR(toolbar), hbox, "", "");
-
-    icon = gtk_image_new_from_stock(GTK_STOCK_JUMP_TO,
-				    GTK_ICON_SIZE_LARGE_TOOLBAR);
-    gtk_toolbar_append_item(GTK_TOOLBAR(toolbar), 
-			    NULL,  _("Go To Page"),
-			    NULL, icon, G_CALLBACK(tg_gui_cb_goto_page), NULL);
-    
-    icon = gtk_image_new_from_stock(GTK_STOCK_GO_BACK,
-				    GTK_ICON_SIZE_LARGE_TOOLBAR);
-    gtk_toolbar_append_item(GTK_TOOLBAR(toolbar), 
-			    NULL,  _("Get Previous Page"),
-			    NULL, icon, G_CALLBACK(tg_gui_cb_prev_page), NULL);
-    
-    icon = gtk_image_new_from_stock(GTK_STOCK_GO_FORWARD,
-				    GTK_ICON_SIZE_LARGE_TOOLBAR);
-    gtk_toolbar_append_item(GTK_TOOLBAR(toolbar), 
-			    NULL, _("Get Next Page"),
-			    NULL, icon, G_CALLBACK(tg_gui_cb_next_page), NULL);
-    
-    icon = gtk_image_new_from_stock(GTK_STOCK_HOME,
-				    GTK_ICON_SIZE_LARGE_TOOLBAR);
-    gtk_toolbar_append_item(GTK_TOOLBAR(toolbar), 
-			    NULL, _("Go to the home page"),
-			    NULL, icon, G_CALLBACK(tg_gui_cb_home), NULL);
-    
-    icon = gtk_image_new_from_stock(GTK_STOCK_MEDIA_PLAY,
-				    GTK_ICON_SIZE_LARGE_TOOLBAR);
-    w = gtk_toggle_button_new();
-    gui->pagebutton = w;
-    gtk_container_add(GTK_CONTAINER(w), icon);
-    g_signal_connect(G_OBJECT(w), "clicked",
-		     G_CALLBACK(tg_gui_cb_toggle_paging), NULL);
-    gtk_toolbar_append_widget(GTK_TOOLBAR(toolbar), w, _("Toggles auto-paging"), NULL);
-
-    /* FIXME */ /*
-    gui->zoomlabel = gtk_label_new(_("100%"));
-    w = gtk_toggle_button_new();
-    gui->zoombutton = w;
-    gtk_container_add(GTK_CONTAINER(w), gui->zoomlabel);
-    g_signal_connect(G_OBJECT(w), "clicked", G_CALLBACK(tg_gui_cb_zoom), NULL);
-    gtk_toolbar_append_widget(GTK_TOOLBAR(toolbar), w, _("Toggles zooming"), NULL);
-    */
-
-    /* 
-    w = gtk_combo_new();
-    gtk_toolbar_append_widget(GTK_TOOLBAR(toolbar), w, _("Selects a channel"), NULL);
-    */
-    gtk_widget_show_all(toolbar);
-    
-    return toolbar;
-}
-
 static void
 tg_gui_init (TgGui *self)
 {
@@ -496,11 +373,11 @@ tg_gui_get_property (GObject *object, guint property_id, GValue *value, GParamSp
 	    break;
 
 	case PROP_CURRENT_CHANNEL:
-	    g_value_set_string (value, self->current_channel);
+	    g_value_set_string (value, self->current_channel_uuid);
 	    break;
 
 	case PROP_ZOOM_FACTOR:
-	    g_value_set_int (value, currentview->zoom_factor);
+	    g_value_set_int (value, self->zoom_factor);
 	    break;
 
 	case PROP_PAGING_ENABLED:
@@ -512,11 +389,11 @@ tg_gui_get_property (GObject *object, guint property_id, GValue *value, GParamSp
 	    break;
 
 	case PROP_CURRENT_PAGE_NUMBER:
-	    g_value_set_int (value, currentview->page_nr);
+	    g_value_set_int (value, self->page_nr);
 	    break;
 
 	case PROP_CURRENT_SUBPAGE_NUMBER:
-	    g_value_set_int (value, currentview->subpage_nr);
+	    g_value_set_int (value, self->subpage_nr);
 	    break;
 
 	default:
@@ -539,21 +416,21 @@ tg_gui_set_property (GObject *object, guint property_id, const GValue *value, GP
 	    break;
 
 	case PROP_CURRENT_CHANNEL:
-	    g_free (self->current_channel);
-	    self->current_channel = g_value_dup_string (value);
-	    channel = tg_gui_channel_find_by_uuid (self->current_channel);
+	    g_free (self->current_channel_uuid);
+	    self->current_channel_uuid = g_value_dup_string (value);
+	    channel = tg_gui_channel_find_by_uuid (self->current_channel_uuid);
 	    if (channel) {
 		tg_gui_channel_select (channel);
 		g_object_unref (G_OBJECT (channel));
 	    } else {
-		if (currentview->channel)
-		    g_object_unref (G_OBJECT (currentview->channel));
-		currentview->channel = NULL;
+		if (self->current_channel)
+		    g_object_unref (G_OBJECT (self->current_channel));
+		self->current_channel = NULL;
 	    }
 	    break;
 
 	case PROP_ZOOM_FACTOR:
-	    currentview->zoom_factor = g_value_get_int (value);
+	    self->zoom_factor = g_value_get_int (value);
 	    break;
 
 	case PROP_PAGING_ENABLED:
@@ -565,11 +442,11 @@ tg_gui_set_property (GObject *object, guint property_id, const GValue *value, GP
 	    break;
 
 	case PROP_CURRENT_PAGE_NUMBER:
-	    currentview->page_nr = g_value_get_int (value);
+	    self->page_nr = g_value_get_int (value);
 	    break;
 
 	case PROP_CURRENT_SUBPAGE_NUMBER:
-	    currentview->subpage_nr = g_value_get_int (value);
+	    self->subpage_nr = g_value_get_int (value);
 	    break;
 
 	default:
@@ -583,7 +460,8 @@ tg_gui_finalize (GObject *object)
 {
     TgGui *gui = TG_GUI (object);
 
-    g_clear_pointer (&gui->app, gtk_widget_destroy);
+    g_clear_object (&gui->builder);
+    g_clear_pointer (&gui->window, gtk_widget_destroy);
     g_clear_object (&gui->settings);
     g_clear_pointer (&gui->channel_children, g_strfreev);
     if (gui->channels != NULL) {
@@ -637,8 +515,8 @@ tg_gui_class_init (TgGuiClass *klass)
     g_object_class_install_property
 	(gobject_class, PROP_PAGING_INTERVAL,
 	 g_param_spec_int ("paging-interval",
-			   _("Paging interval"),
-			   _("Specifies the interval for the auto-pager, in milliseconds."),
+			   "Paging interval",
+			   "Specifies the interval for the auto-pager, in milliseconds.",
 			   1000, 60000, 12000,
 			   G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
@@ -657,40 +535,125 @@ tg_gui_class_init (TgGuiClass *klass)
 			   G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 }
 
+static gint
+tg_gui_update_pixmap(GdkPixbuf *pixbuf)
+{
+    if (pixbuf) {
+	tg_pixpack_load_image(TG_PIXPACK(gui->pixpack), pixbuf);
+    } else {
+	/* no pixbuf, resize to a default and print a warning */
+	g_warning("pixbuf == NULL\n");
+	gtk_widget_set_size_request(GTK_WIDGET(gui->pixpack), 200, 150);
+    }
+
+    return 0;
+}
+
+static gint
+tg_gui_update_page(int *major_nr, int *minor_nr)
+{
+    gint retval;
+    GdkPixbuf *pixbuf;
+    GError *error;
+
+    /* save these and restore them, if necessary */
+    gint old_page = *major_nr;
+    gint old_subpage = *minor_nr;
+
+    /* make http-request, returns the file of the saved thing */
+    retval = tg_http_get_image(gui, gui->current_channel, &pixbuf);
+
+    if (TG_OK == retval) {
+	tg_gui_update_pixmap(pixbuf);
+	g_object_unref(pixbuf);
+	return 0;
+    } else {
+	switch (retval) {
+	    case TG_ERR_PIXBUF:	/* we got an error from the webpage */
+		/* maybe we forgot the subpage nr, or used it when we shouldn't */
+		*minor_nr = (0 == *minor_nr) ? 1 : 0;
+		if (TG_OK != tg_http_get_image(gui, gui->current_channel,
+					       &pixbuf)) { 
+		    if (*minor_nr != 1) {
+			/* maybe we've run out of subpages, go to next main page */
+			*minor_nr = 0;
+			(*major_nr)++;
+			tg_gui_update_entry(*major_nr, *minor_nr);
+			tg_gui_get_the_page(FALSE); /* don't redraw */ 
+			return 0;
+		    } else {
+			tg_gui_print_in_statusbar(
+			    _("Web server error: Wrong page number?"));
+			*major_nr = old_page;  /* restore */
+			*minor_nr = old_subpage;
+			tg_gui_update_entry(*major_nr, *minor_nr);
+			error = NULL;
+			pixbuf = gdk_pixbuf_new_from_resource(
+			    TG_NOTFOUND_PIXMAP, &error);
+			g_assert_no_error(error);
+			tg_gui_update_pixmap(pixbuf);
+			g_object_unref(pixbuf);
+			return -1;
+		    }
+		} else {
+		    tg_gui_update_pixmap(pixbuf);
+		    g_object_unref(pixbuf);
+		    return 0;
+		}		
+	    case TG_ERR_VFS:
+		tg_gui_print_in_statusbar(_("Error making HTTP connection"));
+		return -1;
+	    case TG_ERR_HTTPQUERY:
+		tg_gui_print_in_statusbar(
+		    _("Internal error in HTTP query code"));
+		return -1;
+	    default: 
+		g_assert_not_reached();
+		return -1;
+	}
+    }
+    return 0;
+}
+
 /*******************************
- * return the app gui, with startpage or NULL
+ * return the app gui
  */
 TgGui *
-tg_gui_new (GSettings *settings, gchar *startpage)
+tg_gui_new (GtkApplication *app, GSettings *settings)
 {
-    GtkWidget *toolbar;
     GdkPixbuf *pixbuf;
     GError *error = NULL;
 
     gui = g_object_new (TG_TYPE_GUI, NULL);
 
-    /* the app */
-    gui->app = gnome_app_new (PACKAGE, _("TeleGNOME: Teletext for GNOME"));
-    /* gtk_window_set_policy (GTK_WINDOW (gui->app), FALSE, FALSE, TRUE); */
-    gtk_widget_realize (GTK_WIDGET (gui->app));
+    /* register custom type */
+    g_type_name (TG_TYPE_PIXPACK);
 
-    toolbar = tg_gui_new_toolbar();
+    gui->builder = gtk_builder_new_from_resource (TG_MAIN_UI);
+    gui->window = GTK_WIDGET
+	(gtk_builder_get_object (gui->builder, "main_window"));
+    gtk_application_add_window (app, GTK_WINDOW (gui->window));
+    gui->entry = GTK_WIDGET
+	(gtk_builder_get_object (gui->builder, "page_entry"));
+    gui->pixpack = GTK_WIDGET
+	(gtk_builder_get_object (gui->builder, "pixpack"));
+    gui->progress_bar = GTK_WIDGET
+	(gtk_builder_get_object (gui->builder, "progress_bar"));
+    gui->status_bar = GTK_WIDGET
+	(gtk_builder_get_object (gui->builder, "status_bar"));
 
-    /* attach a keyboard event */
-    g_signal_connect (G_OBJECT (gui->app),
-		      "key_press_event",
-		      G_CALLBACK (tg_cb_keypress), NULL);
-    
-    /* attach the menu */
-    gnome_app_create_menus (GNOME_APP (gui->app), menubar);
+    gtk_builder_add_callback_symbols
+	(gui->builder,
+	 "tg_cb_keypress", G_CALLBACK (tg_cb_keypress),
+	 "tg_gui_cb_goto_page", G_CALLBACK (tg_gui_cb_goto_page),
+	 "tg_gui_cb_prev_page", G_CALLBACK (tg_gui_cb_prev_page),
+	 "tg_gui_cb_next_page", G_CALLBACK (tg_gui_cb_next_page),
+	 "tg_gui_cb_home", G_CALLBACK (tg_gui_cb_home),
+	 "tg_gui_cb_toggle_paging", G_CALLBACK (tg_gui_cb_toggle_paging),
+	 NULL);
+    gtk_builder_connect_signals (gui->builder, NULL);
 
-    gnome_app_add_toolbar (GNOME_APP (gui->app), GTK_TOOLBAR (toolbar),
-			   "nav_toolbar", 0, BONOBO_DOCK_TOP, 2, 0, 0);
-
-    /* the view */
-    currentview = tg_view_new();
-
-    tg_view_set_error_handler(currentview, tg_gui_print_in_statusbar);
+    gui->channel_menu = gtk_application_get_menu_by_id (app, "channels");
 
     gui->settings = g_object_ref (settings);
     g_settings_bind (gui->settings, "channel-children", gui, "channel-children",
@@ -708,22 +671,7 @@ tg_gui_new (GSettings *settings, gchar *startpage)
     g_settings_bind (gui->settings, "current-subpage-number", gui, "current-subpage-number",
 		     G_SETTINGS_BIND_DEFAULT);
 
-    /* the statusbar */
-    gui->statusbar = gnome_appbar_new (TRUE, TRUE, GNOME_PREFERENCES_NEVER);
-    gnome_app_set_statusbar (GNOME_APP (gui->app), gui->statusbar);
-
-    /* make menu hints display on the appbar */
-    gnome_app_install_menu_hints (GNOME_APP (gui->app), menubar);
-
-    /* all the contents */
-    gnome_app_set_contents (GNOME_APP (gui->app),
-			    tg_view_get_widget (currentview));
-
-    g_signal_connect (G_OBJECT (gui->app), "delete_event",
-		      G_CALLBACK (tg_gui_cb_quit),
-		      NULL);
-
-    gtk_widget_show_all (gui->app);
+    gtk_widget_show_all (gui->window);
 
     gui->kb_timer = -1;
     gui->kb_status = INPUT_NEW;
@@ -731,26 +679,16 @@ tg_gui_new (GSettings *settings, gchar *startpage)
     gui->page_progress = 0;
     gui->page_timer = -1;
 
-    gui->progress = gnome_appbar_get_progress(GNOME_APPBAR(gui->statusbar));
-    gtk_progress_bar_set_fraction(gui->progress, 0.0);
-
-#if 0
-    /* the zoom button */
-    /* FIXME */
-    gtk_label_set(GTK_LABEL(gui->zoomlabel), currentview->zoom_factor==1?"100%":"400%");
-    if (currentview->zoom_factor==2) gtk_toggle_button_toggled(GTK_TOGGLE_BUTTON(gui->zoombutton));
-#endif
-
-    /* g_print("Number: %d/%d\n", currentview->page_nr, currentview->subpage_nr); */
+    /* g_print("Number: %d/%d\n", gui->page_nr, gui->subpage_nr); */
 
     gui->channels = NULL;
-    gui->channel_menu = NULL;
 
     tg_gui_refresh_channel_menu();
 
     /* FIXME: */
     /* set the current view, at elem 0 */
-    currentview->channel = tg_gui_channel_find_by_uuid(gui->current_channel);
+    gui->current_channel = tg_gui_channel_find_by_uuid(
+	gui->current_channel_uuid);
 
     tg_gui_update_title_bar();
 
@@ -758,18 +696,17 @@ tg_gui_new (GSettings *settings, gchar *startpage)
        load up the last page we were visiting. Otherwise,
        start with a logo */
     tg_gui_update_entry(0,0);
-    pixbuf = gdk_pixbuf_new_from_file(
-	gnome_program_locate_file(NULL, GNOME_FILE_DOMAIN_PIXMAP,
-				  TG_LOGO_PIXMAP, TRUE, NULL),
-	&error);
-    tg_view_update_pixmap(currentview, pixbuf);
+    error = NULL;
+    pixbuf = gdk_pixbuf_new_from_resource (TG_LOGO_PIXMAP, &error);
+    g_assert_no_error (error);
+    tg_gui_update_pixmap(pixbuf);
     g_object_unref(pixbuf);
     
     /* only auto-change to a page if it was saved the last time */
 
-    if (currentview->page_nr >0 )
-	gui->logo_timer = gtk_timeout_add (TG_LOGO_TIMEOUT, tg_gui_logo_timer,
-					   NULL);
+    if (gui->page_nr > 0)
+	gui->logo_timer = g_timeout_add (TG_LOGO_TIMEOUT, tg_gui_logo_timer,
+					 NULL);
     else
 	gui->logo_timer = -1;
     
@@ -777,9 +714,9 @@ tg_gui_new (GSettings *settings, gchar *startpage)
 }
 
 GtkWidget *
-tg_gui_get_app (TgGui *gui)
+tg_gui_get_window (TgGui *gui)
 {
-    return gui->app;
+    return gui->window;
 }
 
 
@@ -810,21 +747,21 @@ tg_gui_get_the_page (gboolean redraw)
 {
     /* hide the app */
     if (redraw)
-	gtk_widget_hide(GTK_WIDGET(gui->app));
+	gtk_widget_hide(gui->window);
 
     /* stop the logo timer */
     if (gui->logo_timer != -1)
-	gtk_timeout_remove(gui->logo_timer);
+	g_source_remove(gui->logo_timer);
     gui->logo_timer = -1;
 
-    if (currentview->channel)
-	tg_view_update_page(currentview, &currentview->page_nr, &currentview->subpage_nr);
+    if (gui->current_channel)
+	tg_gui_update_page(&gui->page_nr, &gui->subpage_nr);
 
-    tg_gui_update_entry(currentview->page_nr, currentview->subpage_nr);
-    tg_gui_print_in_statusbar ("");
+    tg_gui_update_entry(gui->page_nr, gui->subpage_nr);
+    tg_gui_print_in_statusbar (NULL);
 
     if (redraw) 
-      gtk_widget_show_all (GTK_WIDGET(gui->app));
+      gtk_widget_show_all (gui->window);
 }
 
 
@@ -832,71 +769,57 @@ tg_gui_get_the_page (gboolean redraw)
  * callbacks 
  */
 void
-tg_gui_cb_quit (GtkWidget* widget, gpointer data)
+tg_gui_activate_quit (GSimpleAction *action, GVariant *parameter,
+		      gpointer data)
 {
     /* free the channels */
     if (gui->channels != NULL) {
 	g_slist_free_full(gui->channels, g_object_unref);
 	gui->channels = NULL;
     }
-    tg_view_free(currentview);
+    g_clear_object(&gui->pixpack);
 
     /* get outta here ;) */
-    gtk_main_quit();
+    g_application_quit(g_application_get_default());
 }
 
 void
-tg_gui_cb_about (GtkWidget* widget, gpointer data)
+tg_gui_activate_help_contents (GSimpleAction *action, GVariant *parameter,
+			       gpointer data)
 {
-    static GtkWidget *about;
-    const gchar *authors[]= { "Dirk-Jan C. Binnema <djcb@dds.nl>",
-			      "Arjan Scherpenisse <acscherp@wins.uva.nl>",
-			      "Colin Watson <cjwatson@debian.org>",
-			      NULL    };
+    GError *error;
+    gboolean ret;
 
-    if (about) {
-	gdk_window_show(gtk_widget_get_window(about));
-	gdk_window_raise(gtk_widget_get_window(about));
-	return;
+    error = NULL;
+    ret = gtk_show_uri (gtk_widget_get_screen (gui->window), "ghelp:" PACKAGE,
+			GDK_CURRENT_TIME, &error);
+    if (!ret && error) {
+	g_warning ("Error displaying help: %s", error->message);
+	g_error_free (error);
     }
+}
 
-    about = gtk_about_dialog_new();
-    g_object_set(
-	about,
-	"program-name", PACKAGE,
-	"version", VERSION,
-	"copyright", "\xc2\xa9 1999, 2000 Dirk-Jan C. Binnema, "
-		     "Arjan Scherpenisse; "
-		     "\xc2\xa9 2008 Colin Watson",
-	"comments", _("Teletext for GNOME"),
-	"license", _("GNU General Public License, version 2 or later"),
-	"website", "http://telegnome.sourceforge.net/",
-	"authors", authors,
-	"translator-credits", _("translator-credits"),
-	NULL);
+void
+tg_gui_activate_about (GSimpleAction *action, GVariant *parameter,
+		       gpointer data)
+{
+    GtkDialog *about;
 
-    gtk_window_set_transient_for(GTK_WINDOW(about), GTK_WINDOW(gui->app));
-    gtk_window_set_destroy_with_parent(GTK_WINDOW(about), TRUE);
-
-    g_signal_connect(about, "destroy", G_CALLBACK(gtk_widget_destroyed),
-		     &about);
-    g_signal_connect(about, "response", G_CALLBACK(gtk_widget_destroy),
-		     NULL);
-
-    gtk_widget_show(about);
+    about = GTK_DIALOG (gtk_builder_get_object (gui->builder, "about_dialog"));
+    gtk_dialog_run (about);
+    gtk_widget_hide (GTK_WIDGET (about));
 }
 
 static void
 tg_gui_refresh_timer (void)
 {
-    gdouble perc = gtk_progress_bar_get_fraction(gui->progress);
+    gdouble perc = gtk_progress_bar_get_fraction(GTK_PROGRESS_BAR(gui->progress_bar));
 
-    gui->progress = gnome_appbar_get_progress(GNOME_APPBAR(gui->statusbar));
-    gtk_progress_bar_set_fraction(gui->progress, perc);
+    gtk_progress_bar_set_fraction(GTK_PROGRESS_BAR(gui->progress_bar), perc);
 
     if (gui->page_status) {
-	gtk_timeout_remove(gui->page_timer);
-	gui->page_timer = gtk_timeout_add(gui->page_msecs/100, tg_gui_pager_timer, NULL);
+	g_source_remove(gui->page_timer);
+	gui->page_timer = g_timeout_add(gui->page_msecs/100, tg_gui_pager_timer, NULL);
     }
     
     gui->page_progress =(int)((gui->page_msecs/100)*perc);
@@ -910,42 +833,43 @@ tg_gui_prefs_close_cb (GtkDialog *dialog, gint response_id, gpointer user_data)
 }
 
 void
-tg_gui_cb_preferences (GtkWidget* widget, gpointer data)
+tg_gui_activate_preferences (GSimpleAction *action, GVariant *parameter,
+			     gpointer data)
 {
-    tg_prefs_show(GTK_WINDOW(gui->app), G_CALLBACK(tg_gui_prefs_close_cb));
+    tg_prefs_show(gui, G_CALLBACK(tg_gui_prefs_close_cb));
 }
 
 void
 tg_gui_cb_next_page (GtkWidget* widget, gpointer data)
 {	
-    if (currentview->subpage_nr == 0) 
-	currentview->page_nr++;
+    if (gui->subpage_nr == 0) 
+	gui->page_nr++;
     else
-	currentview->subpage_nr++;
+	gui->subpage_nr++;
     
-    tg_gui_update_entry(currentview->page_nr, currentview->subpage_nr);
+    tg_gui_update_entry(gui->page_nr, gui->subpage_nr);
     tg_gui_get_the_page(FALSE); /* dont redraw */ 
 }
 
 void
 tg_gui_cb_prev_page (GtkWidget* widget, gpointer data)
 {	
-    if (currentview->subpage_nr>0)
-	currentview->subpage_nr--;
-    if (currentview->subpage_nr==0)
-	currentview->page_nr--;
+    if (gui->subpage_nr > 0)
+	gui->subpage_nr--;
+    if (gui->subpage_nr == 0)
+	gui->page_nr--;
     
-    tg_gui_update_entry(currentview->page_nr, currentview->subpage_nr);
+    tg_gui_update_entry(gui->page_nr, gui->subpage_nr);
     tg_gui_get_the_page(FALSE);
 }
 
 void
 tg_gui_cb_home (GtkWidget* widget, gpointer data)
 {	
-    currentview->subpage_nr=0;
-    currentview->page_nr=100;
+    gui->subpage_nr = 0;
+    gui->page_nr = 100;
     
-    tg_gui_update_entry(currentview->page_nr, currentview->subpage_nr);
+    tg_gui_update_entry(gui->page_nr, gui->subpage_nr);
     tg_gui_get_the_page(FALSE);
 }
 
@@ -953,36 +877,17 @@ void
 tg_gui_cb_goto_page (GtkWidget *widget, gpointer data)
 {
     gui->kb_status = INPUT_NEW;
-    if ( -1 == tg_http_get_page_entry (gtk_entry_get_text(GTK_ENTRY(gui->entry)))) {
+    if ( -1 == tg_http_get_page_entry (gui, gtk_entry_get_text(GTK_ENTRY(gui->entry)))) {
 	tg_gui_print_in_statusbar(_("Error in page entry"));
 	return;
     }
     tg_gui_get_the_page(FALSE);
 }
 
-
-/*
- * handler for zoom button
- */
-void 
-tg_gui_cb_zoom (GtkWidget *widget, gpointer data)
-{
-    /* new: just toggle it on click */
-    if (currentview->zoom_factor==1) {
-	gtk_label_set_text(GTK_LABEL(gui->zoomlabel),"400%");
-	currentview->zoom_factor=2;
-    } else if (currentview->zoom_factor==2) {
-	gtk_label_set_text(GTK_LABEL(gui->zoomlabel),"100%");
-	currentview->zoom_factor=1;
-    }		
-    /* now, get the page with the new zoom settings */
-    tg_gui_get_the_page(TRUE);
-}
-
 static gint 
 tg_gui_keyboard_timer (gpointer g) 
 {
-    gtk_timeout_remove(gui->kb_timer);
+    g_source_remove(gui->kb_timer);
     gui->kb_timer = -1;
     gui->kb_status = INPUT_NEW;
     return 0;
@@ -991,9 +896,9 @@ tg_gui_keyboard_timer (gpointer g)
 gint 
 tg_cb_keypress (GtkWidget *widget, GdkEventKey *event)
 {
-    if (event->keyval == GDK_KP_Enter) {
+    if (event->keyval == GDK_KEY_KP_Enter) {
 	tg_gui_cb_goto_page(NULL, NULL);
-	tg_gui_update_entry(currentview->page_nr, currentview->subpage_nr);
+	tg_gui_update_entry(gui->page_nr, gui->subpage_nr);
 	return 0;
     }
     
@@ -1006,9 +911,8 @@ tg_cb_keypress (GtkWidget *widget, GdkEventKey *event)
     }
 
     if (gui->kb_timer != -1)
-	gtk_timeout_remove(gui->kb_timer);
-    gui->kb_timer = gtk_timeout_add(
-	TG_KB_TIMEOUT, tg_gui_keyboard_timer, NULL);
+	g_source_remove(gui->kb_timer);
+    gui->kb_timer = g_timeout_add(TG_KB_TIMEOUT, tg_gui_keyboard_timer, NULL);
     gui->kb_status = INPUT_CONTINUED;
     return 0;
 }
