@@ -28,44 +28,18 @@
 #include <config.h>
 #endif /* HAVE_CONFIG_H */
 
-#include <stdio.h>
-#include <string.h>
+#include <libintl.h>
 
 #include <gio/gio.h>
-#include <gtk/gtk.h>
-#include <libgnome/libgnome.h>
-
-#include "main.h"
-#include "http.h"
-#include "gui.h"
-#include "prefs.h"
-#include "legacy-config.h"
 
+#include "app.h"
 
 int 
 main (int argc, char **argv)
 {
-	TgGui *gui;
-	GSettings *settings;
+    bindtextdomain (GETTEXT_PACKAGE, GNOMELOCALEDIR);
+    bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
+    textdomain (GETTEXT_PACKAGE);
 
-	bindtextdomain(PACKAGE,GNOMELOCALEDIR);
-	textdomain(PACKAGE);
-
-	gnome_program_init (PACKAGE, VERSION, LIBGNOMEUI_MODULE, argc, argv,
-			    NULL);
-
-	settings = g_settings_new ("org.gnome.telegnome");
-	legacy_convert (settings);
-
-	/* build gui, handle cmd line args */
-	if ((argc >1) && (strlen(argv[1])<6)) {
-		gui = tg_gui_new (settings, argv[1]);
-	} else {
-		gui = tg_gui_new (settings, "100");
-	} 
-
-	gtk_widget_show_all (tg_gui_get_app (gui));
-	gtk_main();
-
-	return 0;
+    return g_application_run (G_APPLICATION (tg_app_new ()), argc, argv);
 }
